local math = math
local sqrt = math.sqrt
local deg2rad = math.pi / 180
local rad2deg = 1 / deg2rad

local coords = require( 'saotrace.raygen.coords' )

describe( "delta equatorial", function ()

     local tests = {

	{ ra_s = 0, dec_s = 0,
	  ra_f = 10, dec_f = 0,
	  theta = 10, pa = 90
       },

	{ ra_s = 0, dec_s = 0,
	  ra_f = 0, dec_f = 10,
	  theta = 10, pa = 0
       },

	{ ra_s = 0, dec_s = 0,
	  ra_f = -10, dec_f = 0,
	  theta = 10, pa = -90
       },

	{ ra_s = 0, dec_s = 0,
	  ra_f = 0, dec_f = -10,
	  theta = 10, pa = 180
       },

	{ ra_s = 1, dec_s = 0,
	  ra_f = -1, dec_f = 0,
	  theta = 2, pa = -90,
       },

	{ ra_s = 1, dec_s = 1,
	  ra_f = -1, dec_f = -1,
	  theta = sqrt(8), pa = -135,
	  dtheta = 1e-3, dphi = .01
       },

     }

     for idx,v in ipairs( tests ) do

	local inp = {}

	for k, a in pairs( v ) do
	   inp[k] = a * deg2rad
	end


	local name = string.format( "ra_s = %d, dec_s = %d, r_f = %d, dec_f = %d", inp.ra_s, inp.dec_s,
			  inp.ra_f, inp.dec_f );
	it( name , function ()

                local theta, pa = coords.delta_equatorial( inp.ra_s, inp.dec_s,
                                                          inp.ra_f, inp.dec_f )

                theta = theta * rad2deg
                pa = pa * rad2deg
                assert.near( v.theta, theta, v.dtheta or 0 )
                assert.near( v.pa, pa, v.dphi or 0 )
                end
            )

     end

end
)

