local table = require( 'table' )
local os = require( 'os' )

local srcdir = os.getenv('srcdir') or '.'

local db = table.concat( { srcdir, 'tests', 'data', 'geometry.rdb'}, '/' )


local intrinsic = require( 'saotrace.raygen.mock_intrinsics' )
local giter = require('saotrace.geometry').read{ file = db, mode = 'iterator' }

local va = require('validate.args')

local optic = {}

for geo in giter do

   if not optic[geo.optic] then
      optic[geo.optic] = {}
   end

   optic[geo.optic][geo.shell] = geo

end

local ea = require( 'saotrace.raygen.ea.ideal' ).entrance_aperture


describe( "ea_ideal", function ()

             before_each( function ()
                   getmetatable( require('saotrace.config') ).reset()
                   va.reset()
                   va.opts{ check_spec = true,
                            error_on_bad_spec = true,
                   }
                   intrinsic.stash = {}
             end )

             it( "outermost", function ( )

                    ea{ shell = 1, file = db }

                    assert.same( optic.p[1].rho_f, intrinsic.stash[1][1].opts.ro )
                    assert.same( optic.p[3].rho_f, intrinsic.stash[1][1].opts.ri )

             end)

             it( "middle", function ( )

                    ea{ shell = 3, file = db }

                    assert.same( optic.p[3].rho_f, intrinsic.stash[1][1].opts.ro )
                    assert.same( optic.p[4].rho_f, intrinsic.stash[1][1].opts.ri )

             end)

             it( "inner", function ( )

                    ea{ shell = 6, file = db }

                    assert.same( optic.p[6].rho_f, intrinsic.stash[1][1].opts.ro )
                    assert.same( optic.p[6].rho_f - (optic.p[4].rho_f - optic.p[6].rho_f),
                                  intrinsic.stash[1][1].opts.ri, 'p4-p6' )
             end)

end)
