local intrinsic = require( 'saotrace.raygen.mock_intrinsics' )

local va = require('validate.args')
local vspec = require( 'saotrace.raygen.position.vspec' ):new()
local spec = { vspec:vspec() }

local vobj    = require('saotrace.raygen.validate').vobj:new()
local units   = require('saotrace.raygen.units')
local convert = units.convert

vobj:setopts{ named = false }

local coords = require ('saotrace.raygen.coords')
local min2rad = coords.min2rad
local deg2rad = coords.deg2rad

describe( "position", function ()


    before_each( function ()
          getmetatable( require('saotrace.config') ).reset()
          va.reset()
          va.opts{ check_spec = true,
                   error_on_bad_spec = true,
          }
    end)

    it( "object ", function ( )

           local nspec = vspec:new()

           assert.same( 'raygen', nspec:default() )
           assert.same( 'raygen', nspec:default('elaz') )
           assert.same( 'elaz', nspec:default() )

           assert.error( function () nspec:default('non-existent') end )

           nspec:add( 'foobar', { vtable = { coord =  {enum = 'foobar' } } } )
           assert.same( 'elaz', nspec:default( 'foobar' ) )
           assert.same( 'foobar', nspec:default() )



    end)

    it( "position_default", function  ()

           local ok, val = vobj:validate( spec,
                                          {
                                             theta = 3,
                                             phi = 2,
           })
           assert.is_true( ok )

           assert.same( 3, convert( val.theta, nil, units.angular.arcminute)[1] )
           assert.same( 2, convert( val.phi,   nil, units.angular.degree)[1] )

    end)

    it( "position_raygen", function  ()

           local ok, val = vobj:validate( spec,
                                          {
                                             theta = 7,
                                             phi = 44,
                                             coord = 'raygen'
           })

           assert.is_true( ok )
           assert.near(  7, convert( val.theta, nil, units.angular.arcminute)[1], 1e-6 )
           assert.same( 44, convert( val.phi,   nil, units.angular.degree)[1] )


    end)

    it( "position_osac_elaz", function  ()

           local el = 100
           local az = 300

           local ok, val = vobj:validate( spec,
                                          {
                                             el = el,
                                             az = az,
                                             coord = 'osac-elaz'
           })

           el = el * min2rad
           az = az * min2rad

           local theta, phi = coords.osac_elaz_2_osac_polar( el, az )
           theta, phi = coords.osac_polar_2_raygen_polar( theta, phi  )


           assert.is_true( ok )
           assert.same( theta, convert( val.theta, nil, units.angular.radian )[1], 1e-6 )
           assert.same( phi,   convert( val.phi,   nil, units.angular.radian )[1] )

    end)

end)
