local table = require( 'table' )
local os = require( 'os' )

local srcdir = os.getenv('srcdir') or '.'

local ea_db = table.concat( { srcdir, 'data', 'annulus.rdb'}, '/' )

local intrinsic = require( 'saotrace.raygen.mock_intrinsics' )
local utils = require( 'saotrace.raygen.ea.utils.ring_n_spoke' )
local va = require('validate.args')
local validate = va.validate

describe( "ring_n_spoke dist", function ()
    it( "spokes_wrap_single", function ( )

           local pars = { rmin = 1,
                          rmax = 1,
                          nspokes = 1,
                          nrings = 1,
                          tmin = 0,
                          tmax = 360,
                          dist = 'uniform',
           }

           local spoke = 1
           for spot in utils.ringNspoke( pars ) do

              local exp = pars.tmin;
              assert.same( exp, spot.theta, "spoke " .. spoke )

              spoke = spoke + 1
           end

           assert.same( 2, spoke )

    end)

    it( "spokes_wrap_odd", function ( )

           local pars = { rmin = 1,
                          rmax = 1,
                          nspokes = 5,
                          nrings = 1,
                          tmin = 0,
                          tmax = 360
           }

           local spoke = 1
           for spot in utils.ringNspoke( pars ) do

              local exp = pars.tmin + ( spoke - 1 ) * ( pars.tmax - pars.tmin ) / pars.nspokes
              assert.same( exp, spot.theta, "spoke " .. spoke )

              spoke = spoke + 1
           end

    end)

    it( "spokes_wrap_even", function ( )

           local pars = { rmin = 1,
                          rmax = 1,
                          nspokes = 4,
                          nrings = 1,
                          tmin = 0,
                          tmax = 360
           }

           local spoke = 1
           for spot in utils.ringNspoke( pars ) do

              local exp = pars.tmin + ( spoke - 1 ) * ( pars.tmax - pars.tmin ) / pars.nspokes
              assert.same( exp, spot.theta, "spoke " .. spoke )

              spoke = spoke + 1
           end

    end)

    it( "spokes_bounded_single", function ( )

           local pars = { rmin = 1,
                          rmax = 1,
                          nspokes = 1,
                          nrings = 1,
                          tmin = 0,
                          tmax = 75
           }

           local spoke = 1
           for spot in utils.ringNspoke( pars ) do

              local exp = (pars.tmin + pars.tmax ) / 2
              assert.same( exp, spot.theta, "spoke " .. spoke )

              spoke = spoke + 1
           end

           assert.same( 2, spoke )

    end)

    it( "spokes_bounded_odd", function ( )

           local pars = { rmin = 1,
                          rmax = 1,
                          nspokes = 5,
                          nrings = 1,
                          tmin = 0,
                          tmax = 75
           }

           local spoke = 1
           local last_spot
           for spot in utils.ringNspoke( pars ) do

              local exp = pars.tmin + ( spoke - 1 ) * ( pars.tmax - pars.tmin ) / ( pars.nspokes - 1 )
              assert.same( exp, spot.theta, "spoke " .. spoke )

              spoke = spoke + 1
              last_spot = spot
           end

           assert.same( pars.tmax, last_spot.theta )

    end)


    it( "spokes_bounded_even", function ( )

           local pars = { rmin = 1,
                          rmax = 1,
                          nspokes = 4,
                          nrings = 1,
                          tmin = 0,
                          tmax = 75
           }

           local last_spot
           local spoke = 1
           for spot in utils.ringNspoke( pars ) do

              local exp = pars.tmin + ( spoke - 1 ) * ( pars.tmax - pars.tmin ) / ( pars.nspokes - 1 )
              assert.same( exp, spot.theta, "spoke " .. spoke )

              spoke = spoke + 1
              last_spot = spot
           end

           assert.same( pars.tmax, last_spot.theta )

    end)


    it( "rings_single", function ( )

           local pars = { rmin = 1,
                          rmax = 10,
                          nspokes = 1,
                          nrings = 1,
                          tmin = 0,
                          tmax = 75,
                          dist = 'uniform',
           }

           local ring = 1
           for spot in utils.ringNspoke( pars ) do

              local exp = (pars.rmin + pars.rmax ) / 2
              assert.same( exp, spot.r, "ring " .. ring )

              ring = ring + 1
           end

           assert.same( 2, ring )

    end)

    it( "rings_double", function ( )

           local pars = { rmin = 1,
                          rmax = 10,
                          nspokes = 1,
                          nrings = 2,
                          dist = 'uniform'
           }

           local exp = { pars.rmin, pars.rmax }

           local ring = 1
           for spot in utils.ringNspoke( pars ) do

              assert.same( exp[ring], spot.r )

              ring = ring + 1
           end

           assert.same( 3, ring )

    end)


    it( "rings_odd", function ( )

           local pars = { rmin = 1,
                          rmax = 10,
                          nspokes = 1,
                          nrings = 5,
                          tmin = 0,
                          tmax = 75,
                          dist = 'uniform'
           }

           local ring = 1
           local last_spot
           for spot in utils.ringNspoke( pars ) do

              local exp = pars.rmin + ( ring - 1 ) * ( pars.rmax - pars.rmin ) / ( pars.nrings - 1 )
              assert.same( exp, spot.r, "ring " .. ring )

              ring = ring + 1
              last_spot = spot
           end

           assert.same( pars.rmax, last_spot.r )

    end)


    it( "rings_even", function ( )

           local pars = { rmin = 1,
                          rmax = 10,
                          nspokes = 1,
                          nrings = 4,
                          tmin = 0,
                          tmax = 75,
                          dist = 'uniform'
           }

           local last_spot
           local ring = 1
           for spot in utils.ringNspoke( pars ) do

              local exp = pars.rmin + ( ring - 1 ) * ( pars.rmax - pars.rmin ) / ( pars.nrings - 1 )
              assert.same( exp, spot.r, "ring " .. ring )

              ring = ring + 1
              last_spot = spot
           end

           assert.same( pars.rmax, last_spot.r )

    end)


end)
