local table = require( 'table' )
local os = require( 'os' )
local srcdir = os.getenv('srcdir') or '.'
local units = require('saotrace.raygen.units')
local convert = units.convert

local intrinsic = require( 'saotrace.raygen.mock_intrinsics' )
local simple = require( 'saotrace.raygen.source.simple' )

local va = require('validate.args')

describe( "load sources", function ()

    before_each( function ()
          getmetatable( require('saotrace.config') ).reset()
          va.reset()
          va.opts{ check_spec = true,
                   error_on_bad_spec = true,
          }
          intrinsic.stash = {}
    end)



    it( "point", function ( )

           simple.point{ spectrum = 1.49, position = { theta = 2, phi = 9 } }

           assert.same( 'point1', intrinsic.stash[1][1].name )

           local position = intrinsic.stash[2][1].position

           assert.same( 2, convert( position.theta, nil, units.angular.arcminute)[1] )
           assert.same( 9, convert( position.phi, nil, units.angular.degree )[1] )
           assert.same( -1e30, convert( position.z, nil, units.linear.mm )[1] )

           local spectrum = intrinsic.stash[3]

           assert.same( 1.49, spectrum[1].energy )
           assert.same( 'mono', spectrum.func )

    end)

    it( "disk", function ( )

           simple.disk{ spectrum = 1.49, radius = { 5, 'arcsec' },
                        position = { theta = 2, phi = 9}
           }

           local pars = intrinsic.stash[2][1]

           assert.same( 5, convert( pars.radius, nil, units.angular.arcsecond)[1] )

           assert.same( 2, convert( pars.position.theta, nil, units.angular.arcminute)[1] )
           assert.same( 9, convert( pars.position.phi, nil, units.angular.degree )[1] )
           assert.same( -1e30, convert( pars.position.z, nil, units.linear.mm )[1] )

    end)

    it( "load", function ( )

           local srcfile = table.concat( { srcdir, 'tests', 'data', 'sources.xml'}, '/' )

           assert.has_no.errors( function () simple.load_sources( srcfile ) end )

    end)
end)
